@@ -3,7 +3,7 @@
 //! A lightweight framework for profiling functions, geared towards
 //! `no-std` embedded environments. Initialization is very similar
 //! to how the `log` crate is initialized. By default, there is a
-//! no-op profiler that does nothing until you call [set_profiler].
+//! no-op profiler that does nothing until you call [`set_profiler`].
 //! Once your profiler has been installed, your profiling
 //! functionality will be in use.
 //!
@@ -29,8 +29,8 @@
 //!
 //! With the `proc-macros` feature enabled, you can simply annotate
 //! the target function with the procedural macro
-//! [profile_function](embedded_profiling_proc_macros::profile_function).
-//! Note that you must first set your profiler with the [set_profiler]
+//! [`profile_function`](embedded_profiling_proc_macros::profile_function).
+//! Note that you must first set your profiler with the [`set_profiler`]
 //! function.
 //! ```
 //! # #[cfg(feature = "proc-macros")]
@@ -44,22 +44,25 @@
 
 use core::sync::atomic::{AtomicU8, Ordering};
 
-#[cfg(any(feature = "dwt-systick", doc))]
+#[cfg(feature = "dwt-systick")]
 pub mod dwt_systick;
 #[cfg(test)]
 mod mock;
-#[cfg(any(feature = "proc-macros", doc))]
+#[cfg(feature = "proc-macros")]
 pub use embedded_profiling_proc_macros::profile_function;
 
 pub use fugit;
 
+// do the feature gating on a private type so our public documentation is only in one place
 #[cfg(feature = "container-u32")]
-/// The underlying container of our [`Duration`](fugit::Duration)/[`Instant`](fugit::Instant) types
-pub type EPContainer = u32;
+type PrivContainer = u32;
+#[cfg(all(feature = "container-u64", not(feature = "container-u32")))]
+type PrivContainer = u64;
 
-#[cfg(feature = "container-u64")]
-/// The underlying container of our [`Duration`](fugit::Duration)/[`Instant`](fugit::Instant) types
-pub type EPContainer = u64;
+// the `not(feature = "container-u32")` clause is so we can successfully use `--all-features`.
+/// The underlying container of our [`Duration`](fugit::Duration)/[`Instant`](fugit::Instant) types.
+/// Can be either `u32` or `u64`, depending on features. (default: `u32`)
+pub type EPContainer = PrivContainer;
 
 /// Our [`Duration`](fugit::Duration) type, representing time elapsed in microseconds
 pub type EPDuration = fugit::MicrosDuration<EPContainer>;
@@ -90,7 +93,7 @@ pub trait EmbeddedProfiler {
     fn read_clock(&self) -> EPInstant;
 
     /// Optionally reset the clock to zero. This function will be called at the beginning of
-    /// [start_snapshot].
+    /// [`start_snapshot`].
     ///
     /// TODO: not sure if this API is worth while or not.
     fn reset_clock(&mut self) {}
@@ -98,7 +101,9 @@ pub trait EmbeddedProfiler {
     /// Optionally log the snapshot to some output, like a serial port
     fn log_snapshot(&self, _snapshot: &EPSnapshot) {}
 
-    /// Optional function that gets called at the start of the snapshot recording
+    /// Optional function that gets called at the start of the snapshot recording.
+    /// If one would want to very simple profiling, they could use `at_start` and `at_end`
+    /// to simply toggle a GPIO.
     fn at_start(&self) {}
 
     /// Optional function that gets called at the end of the snapshot recording
@@ -108,9 +113,7 @@ pub trait EmbeddedProfiler {
     fn start_snapshot(&mut self) -> EPInstant {
         self.reset_clock();
         self.at_start();
-        let start = self.read_clock();
-
-        start
+        self.read_clock()
     }
 
     /// computes the duration of the snapshot given the start time
@@ -145,8 +148,14 @@ static STATE: AtomicU8 = AtomicU8::new(UNINITIALIZED);
 #[derive(Debug)]
 pub struct SetProfilerError;
 
-/// Sets the profiler. Must be completed with no other threads running
-/// or, on an embedded single core environment, with interrupts disabled.
+/// Sets the global profiler.
+///
+/// # Safety
+/// Must be completed with no other threads running
+/// or, in an embedded single core environment, with interrupts disabled.
+///
+/// # Errors
+/// returns `Err(SetProfilerError)` when a global profiler has already been configured
 pub unsafe fn set_profiler(
     profiler: &'static dyn EmbeddedProfiler,
 ) -> Result<(), SetProfilerError> {
@@ -174,7 +183,7 @@ pub fn end_snapshot(start: EPInstant, name: &'static str) -> EPSnapshot {
 
 /// Logs the given snapshot with the globally configured profiler
 pub fn log_snapshot(snapshot: &EPSnapshot) {
-    unsafe { PROFILER }.log_snapshot(snapshot)
+    unsafe { PROFILER }.log_snapshot(snapshot);
 }
 
 /// Profiles the given closure `target` with name `name`.
